@@ -57,7 +57,7 @@ This crate includes two [`linalg::LinAlgEx`] implementors:
 
 ```toml
 [dependencies.totsu]
-version = "0.8.0"
+version = "0.9.0"
 features = ["f64lapack"]
 ```
 
@@ -71,7 +71,7 @@ Use this in `Cargo.toml`:
 
 ```toml
 [dependencies.totsu]
-version = "0.8.0"
+version = "0.9.0"
 default-features = false
 features = ["libm"]
 ```
@@ -95,7 +95,7 @@ use totsu::prelude::*;
 use totsu::operator::MatBuild;
 use totsu::problem::ProbQP;
 
-//env_logger::init();
+//env_logger::init(); // Use any logger crate as `totsu` uses `log` crate. 
 
 type LA = FloatGeneric<f64>;
 type AMatBuild = MatBuild<LA, f64>;
@@ -167,6 +167,9 @@ extern crate std;
 pub mod solver; // core, Float
 mod utils;
 
+#[cfg(feature = "std")]
+mod solver_error;
+
 pub mod linalg;
 pub mod operator;
 pub mod cone;
