@@ -27,15 +27,25 @@ where L: LinAlgEx<F>, F: Float
 
     fn op(&self, alpha: F, x: &[F], beta: F, y: &mut[F])
     {
-        // y = a*vec_f*x + b*y;
+        // y = a*vec_c*x + b*y;
         self.vec_c.op(alpha, x, beta, y);
     }
 
     fn trans_op(&self, alpha: F, x: &[F], beta: F, y: &mut[F])
     {
-        // y = a*vec_f^T*x + b*y;
+        // y = a*vec_c^T*x + b*y;
         self.vec_c.trans_op(alpha, x, beta, y);
     }
+
+    fn absadd_cols(&self, tau: &mut[F])
+    {
+        self.vec_c.absadd_cols(tau);
+    }
+
+    fn absadd_rows(&self, sigma: &mut[F])
+    {
+        self.vec_c.absadd_rows(sigma);
+    }
 }
 
 //
@@ -93,6 +103,22 @@ where L: LinAlgEx<F>, F: Float
         self.symmat_f.trans_op(alpha, x_sk, beta, y);
         self.mat_a.trans_op(alpha, x_p, F::one(), y);
     }
+
+    fn absadd_cols(&self, tau: &mut[F])
+    {
+        self.symmat_f.absadd_cols(tau);
+        self.mat_a.absadd_cols(tau);
+    }
+
+    fn absadd_rows(&self, sigma: &mut[F])
+    {
+        let (_n, sk, p) = self.dim();
+
+        let (sigma_sk, sigma_p) = sigma.split2(sk, p).unwrap();
+
+        self.symmat_f.absadd_rows(sigma_sk);
+        self.mat_a.absadd_rows(sigma_p);
+    }
 }
 
 //
@@ -150,6 +176,22 @@ where L: LinAlgEx<F>, F: Float
         self.symvec_f_n.trans_op(-alpha, x_sk, beta, y);
         self.vec_b.trans_op(alpha, x_p, F::one(), y);
     }
+
+    fn absadd_cols(&self, tau: &mut[F])
+    {
+        self.symvec_f_n.absadd_cols(tau);
+        self.vec_b.absadd_cols(tau);
+    }
+
+    fn absadd_rows(&self, sigma: &mut[F])
+    {
+        let (_n, sk, p) = self.dim();
+
+        let (sigma_sk, sigma_p) = sigma.split2(sk, p).unwrap();
+
+        self.symvec_f_n.absadd_rows(sigma_sk);
+        self.vec_b.absadd_rows(sigma_p);
+    }
 }
 
 //
